<?php
defined('C5_EXECUTE') or die('Access Denied.');

use Concrete\Core\Page\Page;
use Concrete\Core\Support\Facade\Url;
use JtFResources\Utilities\Context;

/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This package contains software copyright and proprietary to John Liddiard
 *
 */


/*
 * There are so many variations of how sites could display this data, we are not going to bother with
 * making the format fully configurable. Instead, we make it easy to create block templates.
 *
 * For those developing templates
 * Change these classes to style rows/cells for completed forms
 *
 * See thumbs_up_down_table for an example of easy styling a template.
 */

$submitted_class = empty($submitted_class) ? 'text-success' : $submitted_class;
$missing_class = empty($missing_class) ? 'text-danger' : $missing_class;

/*
$submitted_icon = '<i class="text-success fa fas fa-lg fa-thumbs-up"></i>';
$missing_icon = '<i class="text-danger fa fas fa-lg fa-thumbs-down"></i>';
*/

$submitted_icon = empty($submitted_icon) ? '<i class="text-success fa far fa-lg fa-check-square"></i>' : $submitted_icon;
$missing_icon = empty($missing_icon) ? '<i class="text-danger fa far fa-lg fa-times"></i>' : $missing_icon;

/*
 * IMPORTANT - these include bs5 and bs3 classes for back compatibility
 * https://getbootstrap.com/docs/3.4/components/#progress
 * https://getbootstrap.com/docs/5.3/components/progress/#how-it-works
 */
$progress_done_class = empty($progress_done_class) ? 'bg-success progress-bar-success' : $progress_done_class;
$progress_missing_class = empty($progress_missing_class) ? 'bg-success progress-bar-success progress-bar-striped progress-bar-animated active' : $progress_missing_class;
$progress_empty_class = empty($progress_empty_class) ? 'bg-danger progress-bar-danger progress-bar-striped progress-bar-animated active' : $progress_empty_class;

/*
 * $results is a list of
 * [
 * 'cid'       => // collection ID of the form page
 * 'page'      => // page object
 * 'form_name' => // form name, such as form_reform,
 * 'submitted' => // false for not submitted, otherwise the submitted form entry
 * ]
 */


$results = $controller->getResults();
if (!empty($results)) {
    if (!empty($heading)) {
        ?>
        <h2>
            <?php
            echo $heading ?? '';
            ?>
        </h2>
        <?php
    }


    /*
     * Progress bar
     */
    if (!empty($progress_option)) {
        $done = array_filter($results, function ($a) {
            return !empty($a['submitted']);
        });
        $done_count = count($done);
        $total_count = count($results);
        $percent = 100 * $done_count / $total_count;
        if ($done_count < $total_count) {
            $progress_class = $progress_missing_class;
        } else {
            $progress_class = $progress_done_class;
        }

        if (strpos($progress_option, 'bar') !== false) {
            if (empty($done_count)) {
                /*
                 * Special case, nothing done so show a big red 0% bar
                 */
                ?>
                <div class="progress">
                    <div class="progress-bar <?php echo $progress_empty_class; ?>" role="progressbar"
                         style="width: 100%"
                         aria-valuenow="0"
                         aria-valuemin="0"
                         aria-valuemax="100">
                        <strong>
                            <?php
                            if (strpos($progress_option, 'fraction') !== false) {
                                echo $done_count . ' / ' . $total_count;
                            } else {
                                echo('0%');
                            }
                            ?>
                        </strong>
                    </div>
                </div>
                <?php
            } else {
                /*
                 * Usual case, at least something done so show a green bar or what is done up to 100%
                 */
                ?>
                <div class="progress">
                    <div class="progress-bar <?php echo $progress_class; ?>" role="progressbar"
                         style="width: <?php echo $percent; ?>%"
                         aria-valuenow="<?php echo $done_count; ?>"
                         aria-valuemin="0"
                         aria-valuemax="100">
                        <strong>
                            <?php
                            if (strpos($progress_option, 'fraction') !== false) {
                                echo $done_count . ' / ' . $total_count;
                            } else {
                                echo ((int)$percent) . '%';
                            }
                            ?>
                        </strong>
                    </div>
                </div>
                <?php
            }

        }
    }

    /*
     * Table of individual form status
     */
    ?>
    <table class="table table-striped form-reform-completion-table">
        <thead>
        <tr>
            <th>
                <?php
                echo $form_page_column_label ?? '';
                ?>
            </th>
            <th class="col-4 col-sm-4">
                <?php
                echo $status_column_label ?? '';
                ?>
            </th>
        </tr>
        </thead>
        <tbody>
        <?php
        foreach ($results as $result) {
            /*
             * Work out the submitted/missing variation first
             */
            unset($url);
            if (!empty($result['submitted'])) {
                $form_status_classes = $submitted_class;
                $form_status_icon = $submitted_icon;
                if (!empty($link_to_form_page) && strpos($link_to_form_page, 'done') !== false) {
                    $url = Url::to($result['page']);
                }
                $message = $completed_message ?? null;
            } else {
                $form_status_classes = $missing_class;
                $form_status_icon = $missing_icon;
                if (!empty($link_to_form_page) && strpos($link_to_form_page, 'missing') !== false) {
                    $url = Url::to($result['page']);
                }
                $message = $incomplete_message ?? null;
            }

            /*
             * Then our rows are fairly simple
             */
            ?>
            <tr>
                <td class="<?php echo $form_status_classes; ?>">
                    <?php
                    if (!empty($url)) {
                        ?>
                        <a href="<?php echo $url; ?>">
                            <?php
                            echo $result['page']->getCollectionName();
                            ?>
                        </a>
                        <?php
                    } else {
                        echo $result['page']->getCollectionName();
                    }
                    ?>
                </td>
                <td class="<?php echo $form_status_classes; ?>">
                    <?php
                    echo $form_status_icon ?? '';
                    if (!empty($form_status_icon) && !empty($message)) {
                        echo ' &nbsp; ';
                    }
                    echo $message ?? '';
                    ?>
                </td>
            </tr>
            <?php
        }
        ?>
        </tbody>
    </table>
    <?php

} elseif (Context::in_edit_or_admin()) {
    View::element(
        '/form_reform/blocks/edit/edit_mode_marker',
        [
            'block_name' => $controller->getBlockTypeName(),
            'alert'      => t('No pages/forms configured'),
            'level'      => 'danger',
        ],
        'jl_form_reform');
}
