<?php

namespace Concrete\Package\JlFormReformDisplay\Block\JlFormReformListCookieResults;

defined('C5_EXECUTE') or die('Access Denied.');


use Concrete\Core\Page\Page;
use JtF\FormReform\Store\FormFileStore;
use JtF\FormReform\Store\FormPageAttributeStore;
use JtF\FormReform\Store\FormUserAttributeStore;
use JtF\FormReformDisplay\DisplayBlockController;
use JtF\FormReform\Store\FormCookieStore;
use JtF\FormReform\Store\FormSessionStore;
use JtF\FormReformDisplay\Reports\ArrayStoreLister;
use JtF\FormReformDisplay\Reports\ListMakerDefault;


/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This package contains software copyright and proprietary to John Liddiard
 *
 */

class Controller extends DisplayBlockController
{

    protected $btTable = 'btJlFormReformListCookieResults';

    protected $number_results;
    protected $paginate;
    protected $store;
    protected $attr_handle;

    public function getBlockTypeName()
    {
        return t('List Cookie, Session, Attribute, File Result');
    }

    public function getBlockTypeDescription()
    {
        return t('List form results from the Form Reform cookie, session, attribute, CSV or file stores.');
    }


    public function on_before_render()
    {
        parent::on_before_render();
        $this->set('metadata_columns', ListMakerDefault::metadata_columns(true)); //user specific

        if (!empty($this->form_name) && !empty($this->store)) {
            $results = $this->getResults();
            $this->set('results', $results);
        }
    }

    public function validate($args)
    {
        $errors = $this->app->make('helper/validation/error');
        switch ($args['store'] ?? null) {

            case 'user_attr':
                if (empty($args['attr_handle'])) {
                    $errors->add('Select a user attribute');
                }
                break;

            case 'page_attr':
                if (empty($args['page_attr_handle'])) {
                    $errors->add('Select a page attribute');
                }
                break;

            case 'file':
                if (empty($args['fID'])) {
                    $errors->add('Select a file');
                }
                break;

            case 'cookie':
            case 'session':
                break;
            default:
                $errors->add('Select a form store');
                break;
        }
        return $errors;
    }

    public function save($args)
    {
        if (!empty($args['store']) && $args['store'] === 'page_attr') {
            $args['attr_handle'] = $args['page_attr_handle'] ?? null;
        }
        parent::save($args);
    }

    public function getResults()
    {
        switch ($this->store ?? null) {
            case 'session':
                $store = new FormSessionStore();
                unset($this->filter_configuration['uid']);
                break;

            case 'user_attr':
                if (class_exists('JtF\FormReform\Store\FormUserAttributeStore') && !empty($this->attr_handle)) {
                    $store = new FormUserAttributeStore($this->attr_handle ?? null);
                    unset($this->filter_configuration['uid']);
                }
                break;

            case 'page_attr':
                if (class_exists('JtF\FormReform\Store\FormPageAttributeStore') && !empty($this->attr_handle)) {
                    if ($this->filter_configuration['page_filter_mode'] === 'selected' && !empty($this->filter_configuration['cid'])) {
                        $page = Page::getByID($this->filter_configuration['cid']);
                        if (is_object($page)) {
                            $store = new FormPageAttributeStore($this->attr_handle ?? null, $page);
                        }
                    }
                    if (empty($store)) { // default to current page
                        $store = new FormPageAttributeStore($this->attr_handle ?? null);
                    }
                    /*
                     * We have results for a page, no need to further filter by cid
                     * But we do want to filter by current user
                     * TODO - do we want an option for all users? this is a front end list, so maybe not.
                     *  If anyone wants this, they can code a block and leave out the uid filter setting.
                     */
                    unset($this->filter_configuration['cid']);
                    $this->filter_configuration['uid'] = 'current';
                }
                break;

            case 'file':
                /*
                 * We have specified a file, so any decision about user and page has already
                 * been made
                 */
                if (class_exists('JtF\FormReform\Store\FormFileStore')) {
                    $store = new FormFileStore($this->fID ?? null);
                    unset($this->filter_configuration['cid']);
                    unset($this->filter_configuration['uid']);
                }
                break;

            case 'csv':
                /*
                 * Filter CSV by current user. Any cid filter comes from the configuration.
                 * TODO - do we want an option for all users? this is a front end list, so maybe not.
                 *  If anyone wants this, they can code a block and leave out the uid filter setting.
                 */
                if (class_exists('JtF\FormReform\Store\FormFileStore')) {
                    $store = new FormFileStore($this->fID ?? null);
                    $this->filter_configuration['uid'] = 'current';
                }
                break;

            case 'cookie':
            default:
                $store = new FormCookieStore();
                unset($this->filter_configuration['uid']);
                break;
        }

        if (empty($store)) {
            return;
        }

        $my_forms = $store->getFormsFromStore();
        $my_forms = array_values(array_reverse($my_forms));

        $lister = new ArrayStoreLister($my_forms,
                                       $this->filter_configuration ?? null,
                                       $this->form_name ?? null
        );

        $results = $lister->get_listing($this->order_by ?? null, 0, $this->number_results ?? 1);
        return $results;
    }


}
