<?php
defined('C5_EXECUTE') or die('Access Denied.');

use JtF\FormReform\Reports\FormReformDefaultList;
use JtF\FormReform\FormState\FormReformForm;

/*
 * Software by John Liddiard (aka JohntheFish)
 * www.c5magic.co.uk
 *
 * This package contains software copyright and proprietary to John Liddiard
 *
 */


/*
 * For compatibility between UCP plugin and
 * block dialogue
 */
extract($controller->getSets());
$condition_options = FormReformDefaultList::json_condition_operations();
?>
<div id="filter_config_container" class="form-group">
    <table class="table table-bordered table-striped table-condensed">
        <thead>
        <tr>
            <th>
                <?php
                echo t('Saved form field');
                ?>
            </th>
            <th>
                <?php
                echo t('Comparison');
                ?>
            </th>
            <th>
                <?php
                echo t('Value to compare');
                ?>
            </th>
            <th>
                <a href="#add_data_item" class="add_data_item btn btn-success btn-sm"
                   title="<?php echo t('Add'); ?>">
                    <i class="fa fas fa-plus"></i>
                </a>
            </th>
        </tr>
        </thead>
        <tbody class="sortable_body">
        <?php

        /*
         * Extract from $json_filter_configuration or to get a
         * saved_form_field
         * json_comparison
         * json_comparison
         */

        if (empty($json_filter_configuration)) {
            $json_filter_configuration[0]['saved_form_field'] = '';
            $json_filter_configuration[0]['json_comparison'] = '';
            $json_filter_configuration[0]['case_invariant'] = null;
            $json_filter_configuration[0]['value_to_compare'] = '';
        }

        foreach ($json_filter_configuration as $k => $v) {
            ?>
            <tr class="json_data_filter_row" <?php
            if (empty($v['saved_form_field'])) {
                echo ' style="display:none"';
            } ?>>
                <td class="saved_form_field">
                    <?php
                    echo $form->text('json_filter_configuration[' . $k . '][saved_form_field]',
                                     $v['saved_form_field'] ?? null,
                                     [
                                         'maxlength'   => 255,
                                         'placeholder' => t('Saved form field'),
                                     ]
                    );
                    ?>
                </td>
                <td class="json_comparison">
                    <?php
                    echo $form->select('json_filter_configuration[' . $k . '][json_comparison]',
                                       $condition_options,
                                       $v['json_comparison'] ?? '='
                    );

                    echo $form->checkbox('json_filter_configuration[' . $k . '][case_invariant]',
                                       1,
                                       !empty($v['case_invariant'])
                    );
                    echo ' ';
                    echo $form->label('json_filter_configuration[' . $k . '][case_invariant]', t('Case invariant'))
                    ?>
                </td>
                <td class="value_to_compare">
                    <?php
                    echo $form->text('json_filter_configuration[' . $k . '][value_to_compare]',
                                     $v['value_to_compare'] ?? null,
                                     [
                                         'maxlength'   => 255,
                                         'placeholder' => t('Value to compare'),
                                     ]
                    );
                    ?>
                </td>

                <td>
                    <a href="#sort_data_item" class="sort_data_item btn btn-secondary btn-sm"
                       title="<?php echo t('Drag to sort'); ?>"
                       style="cursor: move;cursor: grab;cursor: -moz-grab;cursor: -webkit-grab;"
                    >
                        <i class="fa fas fa-arrows-v fa-arrows-alt-v"></i>
                    </a>
                    <a href="#remove_data_item" class="remove_data_item btn btn-danger btn-sm"
                       title="<?php echo t('Remove'); ?>">
                        <i class="fa fas fa-trash"></i>
                    </a>
                </td>
            </tr>
            <?php
        }


        ?>
        </tbody>
    </table>
    <div class="alert alert-info">
        <?php
        echo t('Optional filters for the saved form data fields. Multiple filters AND.')
        ?>
    </div>
    <script type="text/javascript">
        (function ($) {
            $(document).ready(function () {
                var filter_config_container = $('#filter_config_container');

                var resolve_indexes = function () {
                    filter_config_container.find('.sortable_body tr').each(function (ix, el) {
                        $(this).find('[name]').each(function () {
                            var this_ctl = $(this);
                            var this_name = this_ctl.attr('name');
                            var new_name = this_name.replace(/\[\d+?\]/, '[' + ix + ']');
                            if (new_name !== this_name) {
                                this_ctl.attr('name', new_name);
                                //console.log(this_name, new_name);
                            }
                        });
                    });
                };
                resolve_indexes();

                /*
                 * always have at least one empty row
                 */
                filter_config_container.on('click', '.remove_data_item', function (ev) {
                    ev.preventDefault();
                    ev.stopPropagation();
                    var this_row = $(this).closest('.json_data_filter_row');
                    if (filter_config_container.find('.json_data_filter_row').length < 2) {
                        var item_inputs = this_row.find('input[type="text"],select');
                        item_inputs.val('').removeAttr('id').removeAttr('value');
                        filter_config_container.find('tr').last().after(this_row);
                        resolve_indexes();
                        this_row.hide();
                        return;
                    }
                    this_row.remove();
                });

                /*
                 * Add a new form input_name. Either copy an existing input, or use a hidden blank input.
                 */
                filter_config_container.on('click', '.add_data_item', function (ev) {
                    ev.preventDefault();
                    ev.stopPropagation();
                    var row_to_copy = filter_config_container.find('.json_data_filter_row').last();
                    if (row_to_copy.is(':visible')) {
                        var new_row = row_to_copy.clone();
                        new_row.find('input').val('');
                        new_row.find('select').val('=');
                        row_to_copy.after(new_row);
                        resolve_indexes();
                        new_row.find('.saved_form_field input').trigger('keyup');
                    } else {
                        row_to_copy.show();
                        resolve_indexes();
                        row_to_copy.find('.saved_form_field input').trigger('keyup');
                    }

                });

                filter_config_container.on('keyup', '.saved_form_field input', function () {
                    var this_input = $(this);
                    var input_handle = make_handle(this_input.val());
                    this_input.val(input_handle);
                    return true;
                });

                /*
                 * Note, additional characters to manage array selectors
                 * These get split out into a series of keys into the array
                 * when the data is extracted (php)
                 */
                var make_handle = function (k) {
                    if (!k || k.length < 1) {
                        return '';
                    }
                    return k.toLowerCase().replace(/[^a-z0-9_\.\[\]]/, '');
                };

                filter_config_container.sortable({
                    axis: 'y',
                    classes: {
                        'json_data_filter_row': 'selected',
                        'metadata_row': 'selected'
                    },
                    containment: 'parent',
                    cursor: 'grabbing',
                    handle: '.sort_data_item',
                    items: '.json_data_filter_row,.metadata_row',
                    scroll: true,
                    helper: function (ev, ui) {
                        ui.width(ui.width());
                        ui.find('td').each(function () {
                            $(this).width($(this).width());
                        })
                        return ui;
                    },
                    stop: function (ev, ui) {
                        filter_config_container.find('.json_data_filter_row,.metadata_row').removeAttr('style');
                        resolve_indexes();
                        return ui;
                    }
                });

            });
        })(jQuery);
    </script>

</div>
