<?php

use Concrete\Core\Package\PackageService;
use JtF\FormReformDisplay\DateFormatter;
use JtF\FormReform\DataValues;
use JtFResources\PluginSystem\PluginNamedSettingsStore;

defined('C5_EXECUTE') or die('Access Denied.');

/*
 * Software by John Liddiard (aka JohntheFish)
 * www.c5magic.co.uk
 *
 * This package contains software copyright and proprietary to John Liddiard
 *
 */
extract($controller->getSets());

$data_values = app()->make(DataValues::class);
[$a, $b] = $data_values->getEmTags();
?>
<fieldset style="margin-top:20px;">
    <legend>
        <?php
        echo t('Date Format');
        ?>
    </legend>
    <div class="form-group row">
        <div class="col-7 col-sm-7">
            <?php
            $last_updated = app()->make(PackageService::class)->getByHandle('jl_last_updated');
            if (!empty($last_updated) && is_object($last_updated)) {
                $named_settings = new PluginNamedSettingsStore('', 'jl_last_updated');
                $named_settings_list = $named_settings->list_all_settings();
                if (empty($named_settings_list)) {
                    ?>
                    <div class="alert alert-warning">
                        <?php
                        echo t('To use %sLast Updated%s to format dates, save your desired settings as a %sNamed Setting%s in the %sLast Updated%s block or %sGlobal Settings%s dashboard. %sSupport%s tab > %sExport%s.',
                               $a, $b,
                               $a, $b,
                               $a, $b,
                               $a, $b,
                               $a, $b,
                               $a, $b,
                        );
                        ?>
                    </div>
                    <?php
                } else {
                    $names_setting_options = ['' => t('--None--')];
                    foreach ($named_settings_list as $v) {
                        $names_setting_options[$v] = $v;
                    }
                }
            } else {
                ?>
                <div class="alert alert-warning">
                    <?php
                    echo t('More date/time formatting options are available by installing %sLast Updated%s.',
                           '<a href="https://marketplace.concretecms.com/marketplace/addons/last-updated1/" target="_blank">', '</a>'
                    );
                    ?>
                </div>
                <?php
            }
            $date_format_text = $date_format ?? '';
            if (!empty($names_setting_options)) {
                if (empty($date_format)) {
                    $date_format_source = 0;
                } elseif (!empty($names_setting_options[$date_format ?? null] ?? null)) {
                    $date_format_source = 1;
                    $date_format_text = '';
                } else {
                    $date_format_source = 0;
                }

                ?>
                <div id="form_display_date_source" class="form-group">
                    <?php
                    echo $form->radio('date_format_source', 1, $date_format_source);
                    echo ' ';
                    echo $form->label('date_format_source', t('Use Last Updated stored setting'));
                    ?>
                    <br>
                    <?php
                    echo $form->radio('date_format_source', 0, $date_format_source);
                    echo ' ';
                    echo $form->label('date_format_source', t('Using PHP Date/Time format string'));
                    ?>
                </div>
                <div id="form_display_date_source_last_updated" class="form-group">
                    <?php
                    echo $form->label('date_format', t('Select a Last Updated named setting'));
                    echo $form->select('date_format', $names_setting_options, $date_format ?? '');
                    ?>
                </div>
                <script type="text/javascript">
                    (function ($) {
                        $(document).ready(function () {
                            var date_source = $('#form_display_date_source input');
                            var source_last_updated = $('#form_display_date_source_last_updated');
                            var source_last_updated_input = source_last_updated.find('select');
                            var source_text = $('#form_display_date_source_text');
                            var source_text_input = source_text.find('input');

                            date_source.on('change', function () {
                                var selected_source = $(this);
                                if (!selected_source.is(':checked')) {
                                    return;
                                }
                                var selected_source_val = parseInt(selected_source.val(), 10);
                                if (selected_source_val > 0) {
                                    source_last_updated.show();
                                    source_last_updated_input.removeAttr('disabled');
                                    source_text.hide();
                                    source_text_input.attr('disabled', 'disabled');
                                } else {
                                    source_text.show();
                                    source_text_input.removeAttr('disabled');
                                    source_last_updated.hide();
                                    source_last_updated_input.removeAttr('disabled');
                                }
                            });
                            date_source.trigger('change');
                        });
                    })(jQuery);
                </script>
                <?php

            }
            ?>
            <div id="form_display_date_source_text">
                <div class="form-group">
                    <?php
                    echo $form->label('date_format', t('Enter a PHP Date/Time format string'));
                    echo $form->text('date_format',
                                     $date_format_text ?? '',
                                     [
                                         'minlength'   => 0,
                                         'maxlength'   => 255,
                                         'placeholder' => DateFormatter::getDefault(),
                                     ]
                    );
                    ?>
                </div>
                <div class="alert alert-info">
                    <?php
                    echo t('Specify a %sPHP DateTime%s format. Leave empty for the default format %s%s%s',
                           '<a href="https://www.php.net/manual/en/datetime.format.php" target="_blank">', '</a>',
                           $a, DateFormatter::getDefault(), $b
                    );
                    ?>
                </div>
            </div>
        </div>

        <div class="col-5 col-sm-5">
            <div class="alert alert-info">
                <?php
                echo t('Configure the %sdate%s display for fields configured in the %sData%s tab.',
                       $a, $b,
                       $a, $b,
                );
                ?>
            </div>
        </div>
    </div>

</fieldset>

