<?php

namespace JtF\FormReformDisplay;

use Concrete\Core\Package\PackageService;

use JtFResources\PluginSystem\PluginNamedSettingsStore;
use DateTime as PHPDateTime;

defined('C5_EXECUTE') or die('Access Denied.');

/*
 * Software by John Liddiard (aka JohntheFish)
 * www.c5magic.co.uk
 *
 * This package contains software copyright and proprietary to John Liddiard
 *
 */

class DateFormatter
{
    const DEFAULT_DATE_TIME_FORMAT = 'Y-m-d H:i:s';

    public static function getDefault()
    {
        return self::DEFAULT_DATE_TIME_FORMAT;
    }

    /*
     * Ideally passed a DateTime object, but can recover from a SQL formatted string.
     */
    public function format($date_time, $format_spec = null)
    {
        if (empty($format_spec)) {
            $format_spec = self::DEFAULT_DATE_TIME_FORMAT;
        }
        $date_time = self::toPHPDateTime($date_time);

        /*
         * When Last Updated is available, we can use that for more advanced display
         */
        if ($date_time instanceof PHPDateTime) {
            $last_updated = app()->make(PackageService::class)->getByHandle('jl_last_updated');
            if (!empty($last_updated) && is_object($last_updated)) {
                $ns = new PluginNamedSettingsStore($format_spec, 'jl_last_updated');
                $ns->ignorePermissions();
                $settings = $ns->getItemAll();
                if (!empty($settings)) {
                    $options = $settings['options'];
                    $plugin = $settings['plugin'];
                    $plugin_manager = app()->make('JtF\LastUpdated\UpdatedPluginsController');
                    $plugin_obj = $plugin_manager->loadPlugin($plugin['item_name']);
                    return $plugin_obj->format(
                        $date_time->format(self::DEFAULT_DATE_TIME_FORMAT),
                        $options,
                        $plugin['item_params']
                    );
                }
            }

            return $date_time->format($format_spec);
        }
    }

    /*
     * This won't cover every possible DT string, but it provides a framework that
     * can be extended as we come across edge cases.
     */
    public static function toPHPDateTime($str)
    {
        if (!is_string($str)) {
            return $str;
        }

        if (self::isDateTime($str) || self::isDate($str)) {
            return new PHPDateTime($str);
        }

        /*
         * A good catchall, but only within timestamp range 1970..2038
         */
        $ts = strtotime($str);
        if (!empty($ts)) {
            $dt = new PHPDateTime();
            $dt->setTimestamp($ts);
            return $dt;
        }

        return $str;
    }


    public static function isDateTime($str)
    {
        if (is_string($str) && preg_match("/^[\d]{4}\-[\d]{2}\-[\d]{2} [\d]{2}:[\d]{2}:[\d]{2}$/", $str)) {
            return true;
        }
    }

    public static function isDate($str)
    {
        if (is_string($str) && preg_match("/^[\d]{4}\-[\d]{2}\-[\d]{2}$/", $str)) {
            return true;
        }
    }
}