<?php

namespace JtF\FormReformDisplay;

defined('C5_EXECUTE') or die('Access Denied.');

use Concrete\Core\Block\BlockController;
use Concrete\Core\Feature\UsesFeatureInterface;
use Concrete\Core\Support\Facade\Config;

use Illuminate\Support\Arr;
use JtF\FormReform\FormHandlers\BlocksInForm;
use JtFResources\SystemBase;
use JtFResources\Utilities\FormNoId;
use JtF\FormReformDisplay\Reports\ListMakerDefault;
use JtF\FormReformDisplay\ResultIDEncryption;

/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This package contains software copyright and proprietary to John Liddiard
 *
 */


if (version_compare(SystemBase::core_version_s(), '9.0.0', 'ge')) {
    abstract class ___FDBlockControllerBase extends BlockController implements UsesFeatureInterface
    {
    }
} else {
    abstract class ___FDBlockControllerBase extends BlockController
    {
    }
}

class DisplayBlockController extends ___FDBlockControllerBase
{
    protected $btCacheBlockRecord                   = false;
    protected $btCacheBlockOutput                   = false;
    protected $btCacheBlockOutputOnPost             = false;
    protected $btCacheBlockOutputForRegisteredUsers = false;

    protected $btInterfaceWidth  = "800";
    protected $btInterfaceHeight = "600";


    protected $form_name;
    protected $filter_configuration;
    protected $all_data;
    protected $no_result_message;
    protected $order_by;
    protected $offset;
    protected $show_data;
    protected $show_data_label;
    protected $show_html;


    public function getBlockTypeDefaultSet()
    {
        return 'form_reform_display';
    }

    public function getRequiredFeatures(): array
    {
        return [];
    }

    public function getDefaultFormName()
    {
        return Config::get('jl_form_reform.default_form_name');
    }

    public function listFormNames()
    {
        return ListMakerDefault::form_names();
    }


    public function getFormNameToList()
    {
        if (empty($this->form_name)) {
            $this->form_name = $this->getDefaultFormName();
        }
        return $this->form_name;
    }


    /*
    * To facilitate common filter options with UCP plugin
     * Callbacks from the 'filter' element.
    */
    public function prefixNamespace($k)
    {
        return 'data[' . $k . ']';
    }

    public function synthesizeData($data)
    {
        return $data;
    }

    /*
  * A lot of what we are doing here is to enable common code between a UCP source
  * plugin and this block edit dialogue and view.
  */

    public function on_start()
    {
        if (version_compare(Config::get('concrete.version'), 9.0, '<')) {
            $this->set('v8', true);
        } else {
            $this->set('v9', true);
        }

        if (!empty($this->filter_configuration) && is_scalar($this->filter_configuration)) {
            $this->filter_configuration = json_decode($this->filter_configuration, true);
            $this->set('filter_configuration', $this->filter_configuration);
        }

        if (!empty($this->show_data) && is_scalar($this->show_data)) {
            $this->show_data = json_decode($this->show_data, true);
            $this->set('show_data', $this->show_data);
        }
        if (!empty($this->show_data_label) && is_scalar($this->show_data_label)) {
            $this->show_data_label = json_decode($this->show_data_label, true);
            $this->set('show_data_label', $this->show_data_label);
        }

        if (!empty($this->show_html) && is_scalar($this->show_html)) {
            $this->show_html = json_decode($this->show_html, true);
            $this->set('show_html', $this->show_html);
        }

        if (!empty($this->json_filter_configuration) && is_scalar($this->json_filter_configuration)) {
            $this->json_filter_configuration = json_decode($this->json_filter_configuration, true);
            $this->set('json_filter_configuration', $this->json_filter_configuration);
        }

        $this->set('app', $this->app);
        $this->set('form', $this->app->make(FormNoId::class));

        self::picker_on_page();
    }


    public function save($args)
    {

        /*
         * Clean up and serialize the filter mapping
         */
        if (!empty($args['data']) && is_array($args['data'])) {
            $args['filter_configuration'] = json_encode($args['data']);
        } else {
            $args['filter_configuration'] = json_encode([]);
        }
        if (isset($args['data'])) {
            unset($args['data']);
        }

        if (empty($args['all_data'])) {
            $args['all_data'] = 0;
        }

        /*
         * Clean up and serialize the data/label mapping
         */
        if (!empty($args['show_data']) && is_array($args['show_data'])) {
            if (array_key_exists('', $args['show_data'])) {
                unset($args['show_data']['']);
            }
            if (array_key_exists(0, $args['show_data']) && empty($args['show_data'][0])) {
                unset($args['show_data'][0]);
            }
            if (array_key_exists('0', $args['show_data']) && empty($args['show_data']['0'])) {
                unset($args['show_data']['0']);
            }
            if (array_key_exists('__proforma__jtf__', $args['show_data'])) {
                unset($args['show_data']['__proforma__jtf__']);
            }
            $args['show_data'] = json_encode($args['show_data']);
        }

        if (!empty($args['show_html']) && is_array($args['show_html'])) {
            if (array_key_exists('', $args['show_html'])) {
                unset($args['show_html']['']);
            }
            if (array_key_exists(0, $args['show_html']) && empty($args['show_html'][0])) {
                unset($args['show_html'][0]);
            }
            if (array_key_exists('0', $args['show_html']) && empty($args['show_html']['0'])) {
                unset($args['show_html']['0']);
            }
            if (array_key_exists('__proforma__jtf__', $args['show_html'])) {
                unset($args['show_html']['__proforma__jtf__']);
            }
            $args['show_html'] = json_encode($args['show_html']);
        }

        if (!empty($args['show_data_label']) && is_array($args['show_data_label'])) {
            $args['show_data_label'] = json_encode($args['show_data_label']);
        }

        /*
         * Serialize the JSON query filter mapping
         */
        if (!empty($args['json_filter_configuration']) && !is_scalar($args['json_filter_configuration'])) {
            $args['json_filter_configuration'] = json_encode($args['json_filter_configuration']);
        }

        parent::save($args);
    }


    public function dataFormatOptions()
    {
        return DisplayItemFormatter::dataFormatOptions();
    }

    public function isDate($str)
    {
        return DateFormatter::isDateTime($str);
    }

    /*
     * ID attribute and extraction, potentially used for a result selection
     * Only relevant if data picker is installed.
     */
    public function id_attr($id)
    {
        if (empty($id) || !self::picker_on_page()) {
            return '';
        }
        return ' data-fr="' . self::id_encode($id) . '"';
    }

    public static function id_encode($id)
    {
        return ResultIDEncryption::id_encode($id);
    }

    public static function id_decode($encoded_id)
    {
        return ResultIDEncryption::id_encode($encoded_id);
    }


    /*
     * An optimised test for having a picker on the page
     * Only need to actually do the work once per view.
     */
    static $picker_on_page = null;

    public static function picker_on_page()
    {
        if (!is_null(self::$picker_on_page)) {
            return self::$picker_on_page;
        }

        if (!class_exists('\JtF\FormReformDataPicker\ResultPickerSources')) {
            self::$picker_on_page = false;
            return self::$picker_on_page;
        }
        $block_list = BlocksInForm::getByNameAndPage(BlocksInForm::ANY_FORM)->getBlocksForForm();
        if (empty($block_list)) {
            self::$picker_on_page = false;
            return self::$picker_on_page;
        }
        foreach ($block_list as $b) {
            $bt_handle = $b->getBlockTypeHandle();
            if ($bt_handle === 'jl_form_reform_result_picker') {
                self::$picker_on_page = true;
                return self::$picker_on_page;
            }
        }
        self::$picker_on_page = false;
        return self::$picker_on_page;
    }
}