<?php

namespace JtF\UCP\ContentSources\Plugins;

use Concrete\Core\Asset\AssetList;
use Concrete\Core\Support\Facade\Config;
use Concrete\Core\View\View;


use JtF\UCP\ContentSources\SourceFormReformDisplayPluginBase;
use JtF\UCP\ContentSources\ContentSourcePluginBase;
use JtF\FormReformDisplay\Reports\ListMakerDefault;

defined('C5_EXECUTE') or die('Access Denied');

/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This software is licensed under the terms described in the ConcreteCMS marketplace.
 * Please find the add-on there for the latest license copy.
 */

/*
 * Intermediary to make sure there are no chicken and egg issues during install.
 * When actually in use, will always inherit from SourceFormReformDisplayPluginBase.
 */
if (class_exists('JtF\UCP\ContentSources\SourceFormReformDisplayPluginBase')) {
    abstract class ___IntermediaryBaseForFormReformDefaultStore extends SourceFormReformDisplayPluginBase
    {
    }
} else {
    abstract class ___IntermediaryBaseForFormReformDefaultStore extends ContentSourcePluginBase
    {
    }
}

class FormReformDefaultStore extends ___IntermediaryBaseForFormReformDefaultStore
{

    public function getName()
    {
        return t('Form Reform Default Store');
    }

    public function getDescription()
    {
        return t('Retrieve form submissions from the Form Reform default store.');
    }

    public function getHelp()
    {
        return $this->getBaseHelp() .
            '<br><br>' .
            $this->getPaginationHelp();
    }


    public function getBaseHelp()
    {
        return t("Configure filters to list form submissions by form name, date, user and page.") . '<br>' . t("Provides content as an array.");
    }


    public function getPaginationHelp()
    {
        $a = '<b>';
        $b = '</b>';
        return t("If the content display plugin supports pagination, enabling %sPaginate at source%s will allow the source to manage pagination before the content is passed (via a transform) to the display.",
                 $a, $b);
    }


    public function edit($controller, $data)
    {
        /*
        * Extract plugin specific data from request data
        * Always need both of these, in this order, or import fails
        */
        extract($controller->getSets());
        $data = $this->synthesizeData($data);
        $form = $this->form();
        ob_start();
        ?>
        <div class="form-group" data-provides="<?php echo h($this->provides()); ?>">
            <h3>
                <?php
                echo $this->getName();
                ?>
            </h3>
            <div class="alert alert-info" style="margin-top:15px;">
                <?php
                echo $this->getDescription();
                echo " ";
                echo $this->getBaseHelp();
                ?>
            </div>
        </div>

        <div class="form-group">
            <?php
            $forms = ListMakerDefault::form_names();
            if (empty($data['form_name'])) {
                $data['form_name'] = Config::get('jl_form_reform.default_form_name');
            }
            echo $form->label('form_name', t('Form Name'));
            echo $form->select($this->prefixNamespace('form_name'),
                               array_combine($forms, $forms),
                               $data['form_name'],
            );
            ?>
        </div>

        <fieldset>
            <legend>
                <?php
                echo t('Filter results');
                ?>
            </legend>
            <?php
            View::element('edit/filter',
                          [
                              'controller' => $controller,
                              'plugin'     => $this,
                              'app'        => $this->app(),
                              'form'       => $form,
                              'data'       => $data,
                          ],
                          'jl_form_reform_display');
            ?>
        </fieldset>

        <fieldset>
            <div class="form-group">
                <?php
                $order_by_options = [
                    'desc_id' => t('Most recent at top'),
                    'asc_id'  => t('Oldest at top'),
                ];

                if (empty($data['sort_order'])) {
                    $data['sort_order'] = 'desc_id';
                }
                echo $form->label($this->prefixNamespace('sort_order'), t('Sort by'));
                echo $form->select($this->prefixNamespace('sort_order'), $order_by_options, $data['sort_order']);
                ?>
            </div>
        </fieldset>

        <fieldset>
            <div class="form-group">
                <?php
                if (empty($data['paginate_source'])) {
                    $data['paginate_source'] = false;
                }
                echo $form->checkbox($this->prefixNamespace('paginate_source'), 'paginate_source',
                                     $data['paginate_source']);
                echo ' ';
                echo $form->label($this->prefixNamespace('paginate_source'), t('Paginate at source'));
                ?>
                <div class="alert alert-info">
                    <?php
                    echo $this->getPaginationHelp();
                    ?>
                </div>
            </div>
        </fieldset>


        <div style="min-height:200px;"></div>
        <?php
        $opts_HTML = ob_get_contents();
        ob_end_clean();
        return $opts_HTML;
    }


    /*
     * Extract
     */
    protected $list;

    public function pull($controller, $data)
    {
        extract($controller->getSets());
        $data = $this->synthesizeData($data);

        if (!empty($data['form_name'])) {
            $lister = $this->app()->make(ListMakerDefault::class, [
                'form_name'            => $data['form_name'],
                'filter_configuration' => $data,
                'number_results'       => 0,
                'order_by'             => $data['sort_order'],
                'paginate'             => $data['paginate_source'],
            ]);
            $this->list = $lister->prepare_list();
            $this->set_source_pagination($controller, $data);
            $results = $this->list->getResults();
        }

        if (!empty($results)) {
            foreach ($results as &$result) {
                $r2 = [
                    'id'            => $result->getId(),
                    'form_name'     => $result->getFormName(),
                    'date_created'  => $result->getDateCreated()->format('Y-m-d H:i:s'),
                    'date_modified' => $result->getDateModified()->format('Y-m-d H:i:s'),
                    'cid'           => $result->getCid(),
                    'uid'           => $result->getUid(),
                    'uid_modified'  => $result->getUidModified(),
                    'ip'            => $result->getIp(),
                    'form_data'     => $result->getFormData(),
                ];
                $result = $r2;
            }

            return $results;
        }
    }

    /*
    * Implement pagination if enabled at source
    */
    protected $max_results;

    public function set_source_pagination($controller, $data)
    {
        extract($controller->getSets());
        $data = $this->synthesizeData($data);

        if (!empty($data['paginate_source']) &&
            $data['paginate_source'] === 'paginate_source' &&
            !empty($content_display_obj) &&
            is_object($content_display_obj)
        ) {
            $display_pagination_settings = $content_display_obj->pagination_settings(
                $controller,
                $content_display_plugin_params ?? null
            );
            if (!empty($display_pagination_settings) &&
                !empty($display_pagination_settings['enable'])
            ) {

                /*
                 * If we are paginating at source, need to count max before setting
                 * pagination
                 */
                $this->max_results = $this->list->getTotalResults();

                $limit = (int)$display_pagination_settings['items'] + (int)$display_pagination_settings['orphan_limit'];
                $page_number = max(1, (int)($pagination_page ?? 1)) - 1;
                $offset = $page_number * (int)($display_pagination_settings['items'] ?? 0);

                $this->list->setLimit($limit + 1); // Note +1, get 1 more than necessary to see if there are more to get!!
                $this->list->setOffset($offset);
            }
        }
    }


    /*
     * Called by paginator in display plugins to detect pagination at source and hence
     * variant behaviour inside pagination.
     */
    public function pagination_at_source($controller, $data)
    {
        extract($controller->getSets());
        $data = $this->synthesizeData($data);
        if (!empty($data['paginate_source']) && $data['paginate_source'] === 'paginate_source') {
            if (!empty($this->max_results)) {
                return max(1, (int)$this->max_results);
            }
            return 1;
        } else {
            return 0;
        }
    }


    /*
     * A query always returns data as a table/array
     */
    public function provides()
    {
        return 'array';
    }
}
